set x=10000
set u=1000
set t=100
set n=10

set name=sample2-R421
echo off
echo Search for small asymmetric loops containing one R421 component
echo (see sample2-R421base.rle)
echo -p 997 (arbitrary prime) just makes sure that only stable conduits are used.
echo .
echo The -c value (202) reflects the minimum repeat rate of the boojum reflector.
echo It might be possible to reduce the total search time by adjusting some of the
echo Aho-Corasick tree and memory settings, but the above shouldn't be too bad.
echo .
echo This batch file shows a two-dimensional search, where variables represent
echo both the amount of time taken and the location of the adjustable component.
echo .
echo There are currently four solutions with the above parameters, and the
echo second one produces a working p993 loop (i=632,j=15: 632-15*4+421=993)
echo The other solutions fail due to overlaps with the R421.  The p993 is
echo larger than the current gun in the gun collection, but it might work as
echo a resettable base loop for a high-period oscillator; it should be possible
echo to insert a glider into the tandem-glider part of the track, plus a piece
echo of junk to absorb the original lead glider.
echo .
echo To adjust the sample2-R421base pattern, move the top 9 still lifes
echo (6 blocks and 3 other objects) to the NE -- the distance to move is
echo given by the j variable value.  When that is done, the ghost Herschels
echo should exactly overlap the ghost Herschels in the output pattern.
echo Adding the appropriate bit to one of the ghost Herschels should then
echo produce a working gun:
echo .
echo ...o.         ...o.
echo ...o.         .o.o.
echo .ooo. becomes .ooo.
echo .o...         .o...
echo .
echo The only caveat is that in search results containing oscillating conduits,
echo it will be necessary to adjust the phase of each oscillator to be the same
echo as in the output pattern at the time that the Herschel reaches the marked
echo 'ghost' location at the beginning of the conduit containing the oscillator.
echo .
echo The boojum reflector in this pattern is also technically adjustable, but
echo since the following glider only cleans up a beehive, adjusting it makes
echo no difference to the loop's period.  There's an R507 variant where the
echo reflector adjustment would potentially require a third variable.
echo .
echo 'L' is the opposite of the 'R' turn in the conduit being tested (R421+8N)
echo because the idea here is to connect the end of the conduit to its beginning,
echo at some period, with stable conduits.  I think that using the appropriate pN
echo conduits for each period requires multiple calls to Hersrch, with a different
echo -p value for each, but a hard number instead of a variable after the 'L('.
echo .
hersrch -p 997 -c 202 -o %name%.rle -n %n% -f %name%.log -s -x %x% -u %u% -t %t% -e (i=600..800,j=0..25)L(i-j*4)[-43-j,43+j]
pause
