// ComponentVariant.h: interface for the ComponentVariant class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_COMPONENTVARIANT_H__8FA15E3A_6FE1_4632_AD54_0D3ADA7FFDD7__INCLUDED_)
#define AFX_COMPONENTVARIANT_H__8FA15E3A_6FE1_4632_AD54_0D3ADA7FFDD7__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#pragma warning(disable: 4786)
#endif // _MSC_VER > 1000

#include "Identifier.h"
#include "CellMap.h"

class ComponentVariant  
{
public:
	void setUseCache();
	void deactivate();
	ComponentVariant(
		char * name,
		long id, 
		long componentId,
		long period,
		long maxCompression,
		long minCompression,
		bool gliderReleased,
		bool gliderRequired,
		bool gliderProhibited,
		CellMap * image,
		CellMap * usage);

	virtual ~ComponentVariant();

	// test if the variant supports given period and compression
	// returns true if yes
	bool query(long period, long compression);

	// check if the current variant doesn't overlap the given variant
	// if it starts at [x,y] and points at direction 'd'
	// returns true if the variants don't overlap
	// uses an internal cache to avoid unnecessary computing
	bool check(long x, long y, Direction d, ComponentVariant * v);

	inline bool getCell(long x, long y) { return 0 != m_image->getCell(x, y); };
	inline bool getUsage(long x, long y) { return 0 != m_usage->getCell(x, y); };
	inline long getId() { return m_id; };
	inline CellMap * getImageMap() { return m_image; }; 
	inline CellMap * getUsageMap() { return m_usage; }; 

	inline void getSize(
		long & xMin,
		long & xMax,
		long & yMin,
		long & yMax)
	{
		xMin = m_xMin[TF];
		xMax = m_xMax[TF];
		yMin = m_yMin[TF];
		yMax = m_yMax[TF];
	}

	inline void getSize(
		Direction dir,
		long & xMin,
		long & xMax,
		long & yMin,
		long & yMax)
	{
		xMin = m_xMin[dir];
		xMax = m_xMax[dir];
		yMin = m_yMin[dir];
		yMax = m_yMax[dir];
	}

	inline bool getGliderReleased() { return m_gliderReleased; };
	inline bool getGliderRequired() { return m_gliderRequired; };
	inline bool getGliderProhibited() { return m_gliderProhibited; };
	inline bool getActive() { return m_active; };

	inline long getComponentId() { return m_componentId; };

	inline const char * getName() { return m_name; };
private:
	char * m_name;
	long m_id;
	long m_componentId;
	long m_maxCompression;
	long m_minCompression;
	long m_period;
	CellMap * m_image;
	CellMap * m_usage;
	bool m_gliderReleased;
	bool m_gliderRequired;
	bool m_gliderProhibited;
	bool m_active;
	bool m_useCache;
	long m_xMin[8];
	long m_xMax[8];
	long m_yMin[8];
	long m_yMax[8];
	struct CheckCacheRecord
	{
		long xMin;
		long width;
		long yMin;
		long height;
		char * array;
	};

	CheckCacheRecord * m_checkCache[8];
};

#endif // !defined(AFX_COMPONENTVARIANT_H__8FA15E3A_6FE1_4632_AD54_0D3ADA7FFDD7__INCLUDED_)
