// ShiftedComponent.cpp: implementation of the ShiftedComponent class.
//
//////////////////////////////////////////////////////////////////////

#include "ShiftedComponent.h"
#include "BasicComponent.h"
#include "ComponentVariant.h"
#include "ComponentRegister.h"
#include "Writer.h"
#include <assert.h>

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

ShiftedComponent::ShiftedComponent()
{
	m_variant = NULL;
	m_variantList = NULL;
	m_variantListLength = 0;
}

ShiftedComponent::~ShiftedComponent()
{
	delete [] m_variantList;
}

void ShiftedComponent::print()
{
	Writer::print(m_variant->getName());
	//m_component->print();
}

bool ShiftedComponent::getCell(long x, long y)
{
	if (m_variant != NULL)
	{
		globalToLocal(m_sx, m_sy, m_sDirection, x, y, &x, &y);
		return m_variant->getCell(x, y);
	}
	return false;
}

bool ShiftedComponent::getUsage(long x, long y)
{
	if (m_variant != NULL)
	{
		globalToLocal(m_sx, m_sy, m_sDirection, x, y, &x, &y);
		return m_variant->getUsage(x, y);
	}
	return false;
}

void ShiftedComponent::getSize(long & xMin, long & xMax, long & yMin, long & yMax)
{
	if (m_variant != NULL)
	{
		m_variant->getSize(m_sDirection, xMin, xMax, yMin, yMax);
	}
	else
	{
		m_component->getMinimumSize(m_sDirection, xMin, xMax, yMin, yMax);
	}
	xMin += m_sx;
	xMax += m_sx;
	yMin += m_sy;
	yMax += m_sy;
}

void ShiftedComponent::getMinimumSize(long & xMin, long & xMax, long & yMin, long & yMax)
{
	m_component->getMinimumSize(m_sDirection, xMin, xMax, yMin, yMax);
	xMin += m_sx;
	xMax += m_sx;
	yMin += m_sy;
	yMax += m_sy;
}

void ShiftedComponent::setVariants(long maxVariants)
{
	m_variantList = new ComponentVariant * [maxVariants];
	assert(m_variantList != NULL);
	m_variantListLength = maxVariants;
	m_numVariants = 0;
}

void ShiftedComponent::loadVariants(BasicComponent *predecessor, BasicComponent *successor)
{
	ComponentRegister * reg = m_component->getRegister();
	long n = m_component->getNumVariants();
	long i;
	m_numVariants = 0;
	for (i = 0; i < n; i++)
	{
		ComponentVariant * var = m_component->getActiveVariant(i);
		if (((predecessor == NULL) || reg->canFollow(predecessor, var))
			&& ((successor == NULL) || reg->canFollow(var, successor)))
		{
			m_variantList[m_numVariants] = var;
			m_numVariants++;
		}
	}
}

void ShiftedComponent::storeVariant()
{
	m_storedVariant = m_variant;
}

void ShiftedComponent::restoreVariant()
{
	m_variant = m_storedVariant;
}

void ShiftedComponent::setVariant(ComponentVariant *var)
{
	m_variant = var;
}

bool ShiftedComponent::variantCheck(ShiftedComponent *sc)
{
	// always use the variant with the greater id as the first one
	// because the one with the lower id has not large enough array
	if (m_variant->getId() >= sc->getVariant()->getId())
	{
		// recalculate the second variant's location
		// to the first variant's local coordinates
		Direction d = decombine(m_sDirection, sc->getSDirection()); // direction
		long x, y;
		globalToLocal(m_sx, m_sy, m_sDirection, sc->getSx(), sc->getSy(), &x, &y); // coordinates
		// so: for the m_variant, the sc's variant starts at x,y and points in direction d
		return m_variant->check(x, y, d, sc->getVariant());
	}
	else
	{
		// recalculate the second variant's location
		// to the first variant's local coordinates
		Direction d = decombine(sc->getSDirection(), m_sDirection); // direction
		long x, y;
		globalToLocal(sc->getSx(), sc->getSy(), sc->getSDirection(), m_sx, m_sy, &x, &y); // coordinates
		// so: for the m_variant, the sc's variant starts at x,y and points in direction d
		return sc->getVariant()->check(x, y, d, m_variant);
	}
}

